/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.enterprise.v3.admin.cluster;



import java.beans.PropertyVetoException;
import java.util.List;
import java.util.logging.Level;

import org.jvnet.hk2.annotations.*;
import org.jvnet.hk2.component.*;
import org.jvnet.hk2.config.*;
import org.glassfish.api.Param;
import org.glassfish.api.ActionReport;
import org.glassfish.api.I18n;
import org.glassfish.api.admin.*;
import org.glassfish.api.admin.config.ReferenceContainer;
import com.sun.enterprise.util.LocalStringManagerImpl;
import com.sun.enterprise.config.serverbeans.*;
import com.sun.enterprise.admin.util.RemoteInstanceCommandHelper;

/**
 *  This is a remote command that deletes a destination config.
 * Usage: delete-config
 configuration_name

 * @author Bhakti Mehta
 */
@Service(name = "delete-config")
@Scoped(PerLookup.class)
@I18n("delete.config.command")
public final class DeleteConfigCommand implements AdminCommand {

    @Param(primary=true)
    String destConfig;

    @Inject
    Configs configs;

    @Inject
    Domain domain;


    final private static LocalStringManagerImpl localStrings =
            new LocalStringManagerImpl(DeleteConfigCommand.class);

    public void execute(AdminCommandContext context) {
        ActionReport report = context.getActionReport();
        report.setActionExitCode(ActionReport.ExitCode.SUCCESS);

        //do not delete default-config
        if (destConfig.equals("default-config") ){
            report.setMessage(localStrings.getLocalString(
                    "Config.defaultConfig", "The default configuration template " +
                            "named default-config cannot be deleted."
                    ));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        //Get the config from the domain
        //does the config exist ?
        //if not return
        final Config config = domain.getConfigNamed(destConfig);
        if (config == null ){
            report.setMessage(localStrings.getLocalString(
                    "Config.noSuchConfig", "Config {0} does not exist.", destConfig));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        // check if the config in use by some other
        // ReferenceContainer -- if so just return...
        List<ReferenceContainer> refContainers = domain.getReferenceContainersOf(config);
        if(refContainers.size() >= 1)  {
            StringBuffer namesOfContainers = new StringBuffer();
            for (ReferenceContainer rc: refContainers)  {
                namesOfContainers.append(rc.getReference()).append(',');
            }
            report.setMessage(localStrings.getLocalString(
                    "Config.inUseConfig", "Config {0} is in use " +
                            "and must be referenced by no server instances or clusters",
                    destConfig,namesOfContainers));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;

        }
        try {
            ConfigSupport.apply(new SingleConfigCode<Configs>() {

                @Override
                public Object run(Configs c) throws PropertyVetoException, TransactionFailure {
                    List<Config> configList = c.getConfig();
                    configList.remove(config);
                    return null;
                }
            }, configs);

        } catch (TransactionFailure ex) {
            report.setMessage(
                    localStrings.getLocalString("Config.deleteConfigFailed",
                            "Unable to remove config {0} ", config) + " "
                            +ex.getLocalizedMessage());
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            report.setFailureCause(ex);
        }

    }

}
